@@ -1,36 +1,42 @@
 #pragma once
 
-#include "../../openpgl_common.h"
-#include "../../data/SampleData.h"
-#include "../../include/openpgl/types.h"
-#include <vector>
 #include <cinttypes>
+#include <vector>
 
+#include "../../data/SampleData.h"
+#include "../../data/SampleStatistics.h"
+#include "../../include/openpgl/types.h"
+#include "../../openpgl_common.h"
 #include "DQT.h"
-#include "Traversal.h"
 #include "Rect.h"
+#include "Traversal.h"
 
-namespace openpgl {
+namespace openpgl
+{
 
-enum LeafEstimator {
+enum LeafEstimator
+{
     REJECTION_SAMPLING = 0,
     PER_LEAF,
 };
 
-enum SplitMetric {
+enum SplitMetric
+{
     MEAN = 0,
     SECOND_MOMENT,
 };
 
-template<typename TDistribution>
-class DirectionalQuadtreeFactory {
-public:
+template <typename TDistribution>
+class DirectionalQuadtreeFactory
+{
+   public:
     const static PGL_DIRECTIONAL_DISTRIBUTION_TYPE DIRECTIONAL_DISTRIBUTION_TYPE = PGL_DIRECTIONAL_DISTRIBUTION_QUADTREE;
 
     using Distribution = TDistribution;
     using Sphere2Square = typename Distribution::Sphere2Square;
 
-    struct Configuration {
+    struct Configuration
+    {
         // Estimator for the incoming radiance (and second moment thereof) of each leaf.
         // REJECTION_SAMPLING
         //   Accumulate samples (contribution / pdf) into quadtree leaves,
@@ -56,20 +62,24 @@ public:
 
         uint32_t maxLevels = 12;
 
-        void serialize(std::ostream& stream) const {
-            stream.write(reinterpret_cast<const char*>(this), sizeof(*this));
+        void serialize(std::ostream &stream) const
+        {
+            stream.write(reinterpret_cast<const char *>(this), sizeof(*this));
         };
 
-        void deserialize(std::istream& stream) {
-            stream.read(reinterpret_cast<char*>(this), sizeof(*this));
+        void deserialize(std::istream &stream)
+        {
+            stream.read(reinterpret_cast<char *>(this), sizeof(*this));
         };
 
-        bool operator==(const Configuration& b) const {
+        bool operator==(const Configuration &b) const
+        {
             return true;
         }
     };
 
-    struct StatsNode {
+    struct StatsNode
+    {
         uint32_t offsetChildren = 0;
 
         float sampleWeight = 0.f;
@@ -80,11 +90,14 @@ public:
         float secondMoment = 0.f;
     };
 
-    struct Statistics {
+    struct Statistics
+    {
         // TODO this is very ugly
-        struct SufficientStatistics {
+        struct SufficientStatistics
+        {
             void applyParallaxShift(const Distribution &dist, const Vector3 shift) {}
-            bool isValid() const { 
+            bool isValid() const
+            {
                 return true;
             }
         } sufficientStatistics;
@@ -92,72 +105,89 @@ public:
         float numSamples = 0;
         std::vector<StatsNode> nodes = {StatsNode()};
 
-        bool isValid() const {
+        bool isValid() const
+        {
             auto valid = [](float number) {
                 return !std::isnan(number) && !std::isinf(number) && number >= 0;
             };
-            for (auto &node : nodes) {
-                if (!valid(node.sampleWeight)) return false;
-                if (!valid(node.splitWeight)) return false;
-                if (!valid(node.numSamples)) return false;
-                if (!valid(node.firstMoment)) return false;
-                if (!valid(node.secondMoment)) return false;
+            for (auto &node : nodes)
+            {
+                if (!valid(node.sampleWeight))
+                    return false;
+                if (!valid(node.splitWeight))
+                    return false;
+                if (!valid(node.numSamples))
+                    return false;
+                if (!valid(node.firstMoment))
+                    return false;
+                if (!valid(node.secondMoment))
+                    return false;
             }
             return nodes[0].sampleWeight > 0 && nodes[0].splitWeight > 0;
         }
 
-        void decay(float &alpha) {
+        void decay(float &alpha)
+        {
             numSamples *= alpha;
-            for (auto &node : nodes) {
+            for (auto &node : nodes)
+            {
                 node.numSamples *= alpha;
                 node.firstMoment *= alpha;
                 node.secondMoment *= alpha;
             }
         }
 
-        void serialize(std::ostream& os) const {
-            os.write(reinterpret_cast<const char*>(&numSamples), sizeof(numSamples));
+        void serialize(std::ostream &os) const
+        {
+            os.write(reinterpret_cast<const char *>(&numSamples), sizeof(numSamples));
             size_t size = nodes.size();
-            os.write(reinterpret_cast<const char*>(&size), sizeof(size));
-            os.write(reinterpret_cast<const char*>(nodes.data()), size * sizeof(nodes[0]));
+            os.write(reinterpret_cast<const char *>(&size), sizeof(size));
+            os.write(reinterpret_cast<const char *>(nodes.data()), size * sizeof(nodes[0]));
         };
 
-        void deserialize(std::istream& is) {
-            is.read(reinterpret_cast<char*>(&numSamples), sizeof(numSamples));
+        void deserialize(std::istream &is)
+        {
+            is.read(reinterpret_cast<char *>(&numSamples), sizeof(numSamples));
             size_t size;
-            is.read(reinterpret_cast<char*>(&size), sizeof(size));
+            is.read(reinterpret_cast<char *>(&size), sizeof(size));
             nodes = std::vector<StatsNode>(size);
-            is.read(reinterpret_cast<char*>(nodes.data()), size * sizeof(nodes[0]));
+            is.read(reinterpret_cast<char *>(nodes.data()), size * sizeof(nodes[0]));
         };
 
         // TODO: Needs to be implmented
-        bool operator==(const Statistics& b) const {
+        bool operator==(const Statistics &b) const
+        {
             return true;
         }
     };
 
-    struct FittingStatistics {
+    struct FittingStatistics
+    {
         uint32_t numSamples = 0;
         uint32_t numNodes = 0;
         uint32_t numSplits = 0;
         uint32_t numMerges = 0;
     };
 
-    void prepareSamples(SampleData* samples, const size_t numSamples, const SampleStatistics &sampleStatistics, const Configuration &cfg) const 
-    {};
+    void prepareSamples(SampleData *samples, const size_t numSamples, const SampleStatistics &sampleStatistics, const Configuration &cfg) const {}
 
-    void fit(Distribution &dist, Statistics &stats, const SampleData* samples, const size_t numSamples, const Configuration &cfg, FittingStatistics &fitStats) {
-        for (uint32_t i = 0; i < 5; i++) {
+    void updateFluenceEstimate(Distribution &dist, const SampleData *samples, const size_t numSamples, const size_t numZeroValueSamples,
+                               const SampleStatistics &sampleStatistics) const
+    {}
+
+    void fit(Distribution &dist, Statistics &stats, const SampleData *samples, const size_t numSamples, const Configuration &cfg, FittingStatistics &fitStats)
+    {
+        for (uint32_t i = 0; i < 5; i++)
+        {
             float decay = 0.25;
             stats.decay(decay);
             update(dist, stats, samples, numSamples, cfg, fitStats);
         }
     }
 
-    void update(Distribution &dist, Statistics &stats, const SampleData* samples, const size_t numSamples, const Configuration &cfg, FittingStatistics &fitStats) {
-        Context ctx = {
-            &dist, &stats, &fitStats, samples, numSamples, &cfg
-        };
+    void update(Distribution &dist, Statistics &stats, const SampleData *samples, const size_t numSamples, const Configuration &cfg, FittingStatistics &fitStats)
+    {
+        Context ctx = {&dist, &stats, &fitStats, samples, numSamples, &cfg};
 
         auto leafEstimator = ctx.cfg->leafEstimator;
         auto splitMetric = ctx.cfg->splitMetric;
@@ -184,23 +214,27 @@ public:
         else if (leafEstimator == LeafEstimator::PER_LEAF && splitMetric == SplitMetric::SECOND_MOMENT && footprintFactor > 0)
             updateInternal<LeafEstimator::PER_LEAF, SplitMetric::SECOND_MOMENT, true>(ctx);
     }
-private:
-    struct Context {
-        Distribution* dist;
-        Statistics* stats;
-        FittingStatistics* fitStats;
-        const SampleData* samples;
+
+   private:
+    struct Context
+    {
+        Distribution *dist;
+        Statistics *stats;
+        FittingStatistics *fitStats;
+        const SampleData *samples;
         const size_t numSamples;
-        const Configuration* cfg;
+        const Configuration *cfg;
     };
 
     // Internal Update Routines
-    template<LeafEstimator TLeafEstimator, SplitMetric TSplitMetric, bool TIsFootprintFactorNonZero>
-    void updateInternal(Context &ctx) {
+    template <LeafEstimator TLeafEstimator, SplitMetric TSplitMetric, bool TIsFootprintFactorNonZero>
+    void updateInternal(Context &ctx)
+    {
         ctx.fitStats->numSamples = ctx.stats->numSamples += ctx.numSamples;
 
         // Accumulate all samples into leaves
-        for (uint32_t i = 0; i < ctx.numSamples; i++) {
+        for (uint32_t i = 0; i < ctx.numSamples; i++)
+        {
             const auto &sample = ctx.samples[i];
 
             OPENPGL_ASSERT(isValid(sample));
@@ -208,19 +242,18 @@ private:
             float firstMoment = 0, secondMoment = 0;
 
             // TODO use if constexpr when we start using c++17
-            if (TLeafEstimator == LeafEstimator::REJECTION_SAMPLING) {
+            if (TLeafEstimator == LeafEstimator::REJECTION_SAMPLING)
+            {
                 firstMoment = sample.weight;
                 secondMoment = sample.weight * sample.weight * sample.pdf;
             }
-            if (TLeafEstimator == LeafEstimator::PER_LEAF) {
+            if (TLeafEstimator == LeafEstimator::PER_LEAF)
+            {
                 firstMoment = sample.weight * sample.pdf;
                 secondMoment = firstMoment * firstMoment;
             }
-
-            Vector3 direction;
-            direction.x = sample.direction.x;
-            direction.y = sample.direction.y;
-            direction.z = sample.direction.z;
+            const pgl_vec3f pglDirection = sample.direction;
+            const Vector3 direction = {pglDirection.x, pglDirection.y, pglDirection.z};
             float footprintFactor = ctx.cfg->footprintFactor;
             splat<TIsFootprintFactorNonZero>(ctx.stats->nodes.data(), footprintFactor, Sphere2Square::directionToPoint(direction), [&](StatsNode &node, float weight) {
                 node.numSamples += weight;
@@ -230,21 +263,26 @@ private:
         }
 
         // Compute sampling and split weights
-        traverse(ctx.stats->nodes.data(),
+        traverse(
+            ctx.stats->nodes.data(),
             [&](uint32_t i, Rect<float> &rect) {
-                return true; // We want to traverse all nodes
+                return true;  // We want to traverse all nodes
             },
             [&](uint32_t i, Rect<float> &rect) {
                 auto &node = ctx.stats->nodes[i];
-                if (node.offsetChildren > 0) {
+                if (node.offsetChildren > 0)
+                {
                     node.sampleWeight = 0;
                     node.splitWeight = 0;
-                    for (uint32_t i = 0; i < 4; i++) {
+                    for (uint32_t i = 0; i < 4; i++)
+                    {
                         StatsNode &child_node = ctx.stats->nodes[node.offsetChildren + i];
                         node.sampleWeight += child_node.sampleWeight;
                         node.splitWeight += child_node.splitWeight;
                     }
-                } else {
+                }
+                else
+                {
                     float n;
                     if (TLeafEstimator == LeafEstimator::REJECTION_SAMPLING)
                         n = 1.0f / ctx.stats->numSamples;
@@ -259,8 +297,7 @@ private:
                     if (TSplitMetric == SplitMetric::SECOND_MOMENT)
                         node.splitWeight = std::sqrt(Sphere2Square::area(rect) * secondMoment);
                 }
-            }
-        );
+            });
 
         // Build new tree according to split weights
         ctx.fitStats->numSplits = 0;
@@ -272,7 +309,8 @@ private:
         ctx.fitStats->numNodes = ctx.stats->nodes.size();
 
         ctx.dist->nodes.clear();
-        for (auto &node : ctx.stats->nodes) {
+        for (auto &node : ctx.stats->nodes)
+        {
             DirectionalQuadtreeNode qnode;
             qnode.offsetChildren = node.offsetChildren;
             qnode.sampleWeight = node.sampleWeight;
@@ -280,37 +318,49 @@ private:
         }
     }
 
-    void buildRecursive(Context &ctx, std::vector<StatsNode> &old_nodes, Rect<float> rect, uint32_t i, uint32_t j, uint32_t level = 0) {
+    void buildRecursive(Context &ctx, std::vector<StatsNode> &old_nodes, Rect<float> rect, uint32_t i, uint32_t j, uint32_t level = 0)
+    {
         StatsNode &old_node = old_nodes[i];
         ctx.stats->nodes[j] = old_node;
-        if (level < ctx.cfg->maxLevels && old_node.splitWeight > (ctx.cfg->splitThreshold * old_nodes[0].splitWeight)) {
+        if (level < ctx.cfg->maxLevels && old_node.splitWeight > (ctx.cfg->splitThreshold * old_nodes[0].splitWeight))
+        {
             ctx.stats->nodes[j].offsetChildren = ctx.stats->nodes.size();
-            for (uint32_t c = 0; c < 4; c++) ctx.stats->nodes.emplace_back();
-            if (old_node.offsetChildren > 0) {
+            for (uint32_t c = 0; c < 4; c++)
+                ctx.stats->nodes.emplace_back();
+            if (old_node.offsetChildren > 0)
+            {
                 for (uint32_t c = 0; c < 4; c++)
                     buildRecursive(ctx, old_nodes, rect.child(c), old_node.offsetChildren + c, ctx.stats->nodes[j].offsetChildren + c, level + 1);
             }
-            else {
+            else
+            {
                 // If we have encountered a leaf node that we want to split,
                 // we continue with buildSplit to potentially further split the node
                 ctx.fitStats->numSplits++;
                 for (uint32_t c = 0; c < 4; c++)
                     buildSplit(ctx, rect.child(c), j, ctx.stats->nodes[j].offsetChildren + c, level + 1);
             }
-        } else {
+        }
+        else
+        {
             ctx.stats->nodes[j].offsetChildren = 0;
             // continue traversal to quantify number of nodes that have been merged
             // TODO switch to disable this
-            traverse(old_nodes.data(),
+            traverse(
+                old_nodes.data(),
                 [&](uint32_t i, Rect<float> &rect) {
-                    if (old_nodes[i].offsetChildren > 0) ctx.fitStats->numMerges++;
+                    if (old_nodes[i].offsetChildren > 0)
+                        ctx.fitStats->numMerges++;
                     return true;
                 },
-                [&](uint32_t i, Rect<float> &rect) { }, i, rect);
+                [&](uint32_t i, Rect<float> &rect) {
+                },
+                i, rect);
         }
     }
 
-    void buildSplit(Context &ctx, Rect<float> rect, uint32_t p, uint32_t i, uint32_t level) {
+    void buildSplit(Context &ctx, Rect<float> rect, uint32_t p, uint32_t i, uint32_t level)
+    {
         const auto &parent = ctx.stats->nodes[p];
         auto &node = ctx.stats->nodes[i];
 
@@ -321,17 +371,19 @@ private:
         node.sampleWeight = parent.sampleWeight / 4;
         node.splitWeight = parent.splitWeight / 4;
 
-        return; // TODO splitting further seems to lead to overfitting
+        return;  // TODO splitting further seems to lead to overfitting
 
-        if (level < ctx.cfg->maxLevels && node.splitWeight > (ctx.cfg->splitThreshold * ctx.stats->nodes[0].splitWeight)) {
+        if (level < ctx.cfg->maxLevels && node.splitWeight > (ctx.cfg->splitThreshold * ctx.stats->nodes[0].splitWeight))
+        {
             ctx.fitStats->numSplits++;
             uint32_t offsetChildren = ctx.stats->nodes.size();
             node.offsetChildren = offsetChildren;
-            for (uint32_t c = 0; c < 4; c++) ctx.stats->nodes.emplace_back();
+            for (uint32_t c = 0; c < 4; c++)
+                ctx.stats->nodes.emplace_back();
             for (uint32_t c = 0; c < 4; c++)
                 buildSplit(ctx, rect.child(c), i, offsetChildren + c, level + 1);
         }
     }
 };
 
-}
\ No newline at end of file
+}  // namespace openpgl
\ No newline at end of file
